@@ -81,7 +81,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, signals => {
     ),
   ];
 
-our $VERSION = '2.1.7';
+our $VERSION = '2.2.0';
 
 my $SANE_NAME_PAGE_HEIGHT = SANE_NAME_PAGE_HEIGHT;
 my $SANE_NAME_PAGE_WIDTH  = SANE_NAME_PAGE_WIDTH;
@@ -343,8 +343,14 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
         if ( $opt->{type} == SANE_TYPE_GROUP or not defined $vbox ) {
             $vbox = Gtk3::VBox->new;
             $vbox->set_border_width( $self->style_get('content-area-border') );
-            my $text =
+            my $text = (
                 $opt->{type} == SANE_TYPE_GROUP
+
+                  # A brother scanner used an empty string as a group title,
+                  # which then results in a tab with no title, which is
+                  # confusing and can be missed, so set to the default.
+                  and $opt->{title} ne $EMPTY
+              )
               ? $d_sane->get( $opt->{title} )
               : __('Scan Options');
             my $scwin = Gtk3::ScrolledWindow->new;
@@ -522,7 +528,8 @@ sub _update_option_visibility {
           : $self->{option_widgets}{ $opt->{name} }->get_parent;
         my $geometry = $self->_geometry_option($opt);
         if ($show) {
-            $container->show_all;
+            $container->show_all;            # show hbox
+            $container->get_parent->show;    # show vbox
 
             # Find associated group
             next if ( $opt->{type} == SANE_TYPE_GROUP );
