@@ -41,7 +41,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '2.1.7';
+our $VERSION = '2.2.0';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -206,8 +206,14 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
         if ( $opt->{type} == SANE_TYPE_GROUP or not defined $vbox ) {
             $vbox = Gtk3::VBox->new;
             $vbox->set_border_width( $self->style_get('content-area-border') );
-            my $text =
+            my $text = (
                 $opt->{type} == SANE_TYPE_GROUP
+
+                  # A brother scanner used an empty string as a group title,
+                  # which then results in a tab with no title, which is
+                  # confusing and can be missed, so set to the default.
+                  and $opt->{title} ne $EMPTY
+              )
               ? $d_sane->get( $opt->{title} )
               : __('Scan Options');
             my $scwin = Gtk3::ScrolledWindow->new;
@@ -462,8 +468,11 @@ sub scan {
         npages               => $npages,
         start                => $start,
         step                 => $step,
-        cancel_between_pages => $self->get('cancel-between-pages'),
-        started_callback     => sub {
+        cancel_between_pages => (
+                  $self->get('cancel-between-pages')
+              and $self->get('available-scan-options')->flatbed_selected
+        ),
+        started_callback => sub {
             if ( $npages == 0 and $self->get('max-pages') > 0 ) {
                 $npages = $self->get('max-pages');
             }
